<?php

/*
 * 	Author:		Jarrod Oberto
 * 	Date:		16 Aug	11
 * 	Purpose:	Create a letter collage
 * 	Website:	www.jarrodoberto.com	
 *	License:	This class is licensed under Creative Commons 3.
 *				http://creativecommons.org/licenses/by-sa/3.0/
 */

class imageCollage {

	private $_iterations = 5000;
	private $_fontSizeMax = 35;
	private $_fontSizeMin = 3;
	private $_font = 'Qarmic_sans_Abridged.ttf';
	private $_bgColorArray = array('r' => 0, 'g' => 0, 'b' => 0);
	private $_charColor = array('r' => 255, 'g' => 255, 'b' => 255);

	public function __construct($canvasWidth, $canvasHeight) 
	{

		// *** Needed for loading the font		
		putenv('GDFONTPATH=' . realpath('.'));

		// *** The character set we will display
		$charArray = range('A', 'Z');

		## --------------------------------------------------------
		#create the canvas

		// *** Create the background canvas - the size of the image we want to generate
		$im = @imagecreatetruecolor($canvasWidth, $canvasHeight);

		// *** Allocate a color for an image
		$backgroundColor = imagecolorallocate($im, $this->_bgColorArray['r'], $this->_bgColorArray['g'], $this->_bgColorArray['b']);

		// *** Fill our canvas with the color
		imagefilledrectangle($im, 0, 0, $canvasWidth, $canvasHeight, $backgroundColor);

		## --------------------------------------------------------
		#the iteration	

		$step = (100 / ($this->_fontSizeMax - $this->_fontSizeMin));
		$step = $step / 2.5;
		$stepCounter = 0;

		while ($this->_fontSizeMax >= $this->_fontSizeMin) {

			$this->_fontSizeMax--;
			$falseCount = 0;

			while ($falseCount < $this->_iterations) {

				$result = $this->process($im, $charArray, $canvasWidth, $canvasHeight);

				if (!$result) {
					$falseCount++;
				}
			}

			$stepCounter += $step;
		}

		## --------------------------------------------------------
		#save the image	

		$this->saveImage('collage.jpg', $im);
	}

	## --------------------------------------------------------

	private function process($im, $charArray, $canvasWidth, $canvasHeight) 
	{

		// *** Get random char from our array
		$charKey = array_rand($charArray);
		$char = $charArray[$charKey];

		// *** Get text width & height
		$textSizeArray = $this->getTextSize($this->_fontSizeMax, 0, $this->_font, $char);
		$textWidth = $textSizeArray['width'];
		$textHeight = $textSizeArray['height'];

		// *** Choose random starting position
		// *** x position is from 0 to the max width less the character width. This is so it doesn't overlap the edge of the image
		// *** y is the same but with the height
		$x = rand(0, $canvasWidth - $textWidth);
		$y = rand(0, $canvasHeight - $textHeight);

		// *** Create a temp image the size of our char, and copy a piece of our canvas to it. 
		// *** we take our canvas and cut out a peice the size of our charater, starting at our random position 
		$imTmp = imagecreatetruecolor($textWidth, $textHeight);
		imagecopyresampled($imTmp, $im, 0, 0, $x, $y, $textWidth, $textHeight, $textWidth, $textHeight);

		// *** The trick to the show is to test if this piece of canvas we cut out has any color OTHER THAN THE BACKGROUND color.				
		$result = $this->isColorVacant($imTmp, $this->_bgColorArray);
		if ($result) {

			// *** Add text
			$this->addChar($im, $char, $x, $y, 0, $this->_charColor, $this->_fontSizeMax, 0, $this->_font);
		}

		return $result;
	}

	## --------------------------------------------------------

	private function isColorVacant($imTmp, $colorToTestArray) 
	{
		$r1 = $colorToTestArray['r'];
		$g1 = $colorToTestArray['g'];
		$b1 = $colorToTestArray['b'];

		for ($y = 0; $y < imagesy($imTmp); $y++) {
			for ($x = 0; $x < imagesx($imTmp); $x++) {
				$rgb = ImageColorAt($imTmp, $x, $y);

				$r = ($rgb >> 16) & 0xFF;
				$g = ($rgb >> 8) & 0xFF;
				$b = $rgb & 0xFF;

				if (($r != $r1) && ($g != $g1) && ($b != $b1)) {
					return false;
				}
			}
		}
		return true;
	}

	## --------------------------------------------------------

	public function addChar($im, $text, $x, $y, $padding = 0, $fontColor = '#fff', $fontSize = 12, $angle = 0, $font = null) 
	{
		// *** Split out the colors from the array	
		$r = $fontColor['r'];
		$g = $fontColor['g'];
		$b = $fontColor['b'];

		// *** Get text font
		//$font = $this->getTextFont($font);
		// *** Get text size
		$textSizeArray = $this->getTextSize($fontSize, $angle, $font, $text);
		$textWidth = $textSizeArray['width'];
		$textHeight = $textSizeArray['height'];

		$y = $y + $textHeight;

		// *** Set text color		
		$fontColor = imagecolorallocate($im, $r, $g, $b);

		// *** Add text
		imagettftext($im, $fontSize, $angle, $x, $y, $fontColor, $font, $text);
	}

	## --------------------------------------------------------

	private function getTextSize($fontSize, $angle, $font, $text) {

		// *** Define box (so we can get the width)
		$box = @imageTTFBbox($fontSize, $angle, $font, $text);

		// ***  Get width of text from dimensions
		$textWidth = abs($box[4] - $box[0]);

		// ***  Get height of text from dimensions (should also be same as $fontSize)
		$textHeight = abs($box[5] - $box[1]);

		return array('height' => $textHeight, 'width' => $textWidth);
	}

	## --------------------------------------------------------

	public function saveImage($savePath, $im, $imageQuality = "100") {
		imagejpeg($im, $savePath, $imageQuality);
	}

	## --------------------------------------------------------

	public function displayImage($im, $imageQuality = "100") {
		header('Content-type: image/jpeg');
		imagejpeg($im, '', $imageQuality);
	}

	## --------------------------------------------------------
}
?>